<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Felix Nagel (info@felixnagel.com)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(PATH_tslib.'class.tslib_pibase.php');

// check for loaded t3jquery extension		
if (t3lib_extMgm::isLoaded('t3jquery')) {
	require_once(t3lib_extMgm::extPath('t3jquery').'class.tx_t3jquery.php');
}


/**
 * Plugin 'Sourcecode (beautyOfCode)' for the 'beautyofcode' extension.
 *
 * @author	Felix Nagel <info@felixnagel.com>
 * @package	TYPO3
 * @subpackage	tx_beautyofcode
 */
class tx_beautyofcode_pi1 extends tslib_pibase {
	var $prefixId      = 'tx_beautyofcode_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_beautyofcode_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'beautyofcode';	// The extension key.
	var $pi_checkCHash = true;
	
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	 
	function main($content,$conf) {
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		
		// init vars
		$this->error = false;
		$content = "";
		
		// get extension manager config
		$this->extManConf = $this->getExtManConf();
		// check for static template
		$this->checkForStaticTempl();
		
		// check for template file
		$this->getTemplateFile();
		
		// proceed if no error is saved
		if (!$this->error) {
			// parse XML data into php array
			$this->pi_initPIflexForm();
			
			// get configs (Ext Manager, Flexform)
			$this->flexFormValue = $this->getflexFormValue();
			// merge Extension Manager configs with TS (if set)
			$this->mergeExtConfWithTS();	
			
			// set Header data
			$this->setHeaderData();		
			
			//create preview
			$this->makePreview();		
			
			// get HTML
			$content = $this->getHTML();
		} else {
			$content = '<div style="text-align: left; text-size: 12px; color: red; margin: 10px; padding: 10px; background: white; border: 3px solid red;"><strong>Beauty of Code Extension Error</strong><br /><p><em>PID: '.$this->cObj->data['pid'].'</em><br /><em>UID: '.$this->cObj->data['uid'].'</em></p><p>'.implode("<br />", $this->error).'</p></div>';
		}
		
		return $content;
	}
	
	/**
	 * Function set header data (JS files and JS HTML)
	 */
	public function setHeaderData() {
		
		// empty header, js needs to be included once
		$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] = null;
		
		// please note only the jquery core js is included by t3jquery. 
		// All other components added manually cause of more flexibility
		if (T3JQUERY === true) {
			// add jQuery core by t3jquery extension
			tx_t3jquery::addJqJS();
		} else {
			// add jQuery core manually if defined
			if ($this->conf['addjQuery']>0) {
				$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] .= '<script src="'.$GLOBALS['TSFE']->tmpl->getFileName("EXT:beautyofcode/res/jquery-1.3.2.min.js").'" type="text/javascript"></script>'."\n";
			}
		}
		
		// add jquery.beautyOfCode.js
		$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] .='<script type="text/javascript" src="'.$this->extManConf['scriptUrl'].'"></script>';
			
		// get defaults
		$defaults = $this->getDefaults();
		
		// choose jQuery function selector
		$jQvar = ($this->conf['noConflict']) ? "jQuery" : "$";
		
		// additional selector? (not supported by JS yet)
		$jQuerySelector = (strlen(trim($this->conf['jQuerySelector']))>0) ? trim($this->conf['jQuerySelector']) . ' ' : false;
		
		// get language strings
		$language = $this->getLanguageStrings();
		
		$jsCode = "\n";
		// add noConflict jQuery code
		if ($this->conf['noConflict']) {
			$jsCode											.= $jQvar.'.noConflict();'."\n";
		}
		$jsCode 											.= $jQvar.'(document).ready(function(){'."\n";
		$jsCode 											.= "\t".$jQvar.'.beautyOfCode.init({'."\n";
		if (!empty($this->extManConf['baseUrl'])) $jsCode 	.= "\t\t".'baseUrl: "'.$this->extManConf['baseUrl'].'",'."\n";
		if (!empty($this->extManConf['scripts'])) $jsCode 	.= "\t\t".'scripts: "'.$this->extManConf['scripts'].'",'."\n";
		if (!empty($this->extManConf['styles'])) $jsCode 	.= "\t\t".'styles: "'.$this->extManConf['styles'].'",'."\n";
		if (strlen(trim($this->conf['theme']))> 0) $jsCode 	.= "\t\t".'theme: "'.trim($this->conf['theme']).'",'."\n";
		if (!empty($this->extManConf['defaults'])) $jsCode 	.= "\t\t".'defaults: {'.$defaults.'},'."\n";
		if ($language) $jsCode 								.= "\t\t".'config: { '.$language."\n\t\t },\n";
		// add a custom jQuery selector
		if ($jQuerySelector) {
			$jsCode 										.= "\t\t".'ready: function() {'."\n";
			$jsCode 										.= "\t\t\t".$jQvar.'("'.trim($this->conf['jQuerySelector']).' pre.code:has(code[class])").beautifyCode();'."\n";
			$jsCode 										.= "\t\t".'},'."\n";
		}
		$jsCode 											.= "\t\t".'brushes: ["Plain"'.$this->getBrushes().']'."\n";
		$jsCode 											.= "\t".'});'."\n";
		$jsCode 											.= '});'."\n";
		
		// add hook to add custom JS in header
		if ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['addJS_setHeaderData']) {
			foreach($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['addJS_setHeaderData'] as $_funcRef) {
				if ($_funcRef) {
					t3lib_div::callUserFunction($_funcRef, $jsCode, $this);
				}
			}
		}
				
		// add js init
		$GLOBALS["TSFE"]->setJS($this->extKey, $jsCode);
	}
	
	/**
	 * Function to set HTML code
	 *
	 * @return	string  contains build html with sourcecode
	 */
	public function getHTML() {	
		// make html
		$code='';
		$lang = ($this->flexFormValue['lang']) ? $this->flexFormValue['lang'] : "plain";	
		$code .= "\n".'<pre class="code"><code class="'.$lang.$this->getCssConfig().'">'."\n";
		$code .= htmlspecialchars($this->flexFormValue['code'])."\n";
		$code .= '</code></pre>'."\n";		
		
		// make label
		$label = ($this->conf['showLabel'] && trim($this->flexFormValue['label']) != "") ? trim($this->flexFormValue['label']) : "";
				
		// Extract subparts from the main template
		$templateMain = $this->cObj->getSubpart($this->templateHtml, '###TEMPLATE_MAIN###');
		
		// only generate label when set
		if ($label != "") {
			// Extract subparts from the label template
			$templateLabel = $this->cObj->getSubpart($this->templateHtml, '###TEMPLATE_LABEL###');			  
			
			// fill label marker array
			$markerArrayLabel['###LABEL###'] = $label; 
			// replace markers in the label template
			$label = $this->cObj->substituteMarkerArray($templateLabel, $markerArrayLabel);
		}
		
		// fill main marker array
		$markerArrayMain['###CODE###'] = $code;		 
		$markerArrayMain['###LABEL###'] = $label;	
		
		// hook to add possibility to add custom marker in template
		if ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['addMarker_getHTML']) {
			foreach($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['addMarker_getHTML'] as $_funcRef) {
				if ($_funcRef) {
					t3lib_div::callUserFunction($_funcRef, $markerArrayMain, $this);
				}
			}
		}
				
		// replace markers in the main template
		$content = $this->cObj->substituteMarkerArray($templateMain, $markerArrayMain);
		
		return $content;
	}
	
	/**
	 * Function to get Extension Manager settings
	 *
	 * @return	array contains extension manager settings
	 */
	public function getExtManConf() {
		$temp = array();
	
		// get values set in extension manager module
		$extManConf = isset($GLOBALS["TYPO3_CONF_VARS"]["EXT"]["extConf"]["beautyofcode"]) ? unserialize($GLOBALS["TYPO3_CONF_VARS"]["EXT"]["extConf"]["beautyofcode"]) : false;

		if (is_array($extManConf)) {	
			// add source file
			$temp['scriptUrl'] = (strlen(trim($extManConf['scriptUrl']))>0) ? $this->makeAbsolutePath(trim($extManConf['scriptUrl'])) : $this->makeAbsolutePath("EXT:beautyofcode/res/jquery.beautyOfCode.js");
			
			// if empty online repo is used
			$temp['baseUrl'] = (strlen(trim($extManConf['baseUrl']))>0) ? t3lib_div::getIndpEnv('TYPO3_SITE_URL').$this->makeAbsolutePath(trim($extManConf['baseUrl'])) : "";
			
			$temp['scripts'] = (strlen(trim($extManConf['scripts']))>0 && trim($extManConf['scripts']) != "scripts/") ? trim($extManConf['scripts']) : "";
			
			$temp['styles'] = (strlen(trim($extManConf['styles']))>0 && trim($extManConf['styles']) != "styles/") ? trim($extManConf['styles']) : "";	
		} else {		
			$this->handleError('Missing Extension Manager configuration. Please check your typo3conf/localconf.php');
		}
		return $temp;
	}
	
	/**
	 * Function to merge TS with Extension Manager configs
	 *
	 */
	public function mergeExtConfWithTS() {		
		if (strlen($this->conf['scriptUrl'])>0) $this->extManConf['scriptUrl'] = $this->makeAbsolutePath(trim($this->conf['scriptUrl']));
		
		if (strlen($this->conf['baseUrl'])>0) $this->extManConf['baseUrl'] = t3lib_div::getIndpEnv('TYPO3_SITE_URL').$this->makeAbsolutePath(trim($this->conf['baseUrl']));
		
		if (strlen(trim($this->conf['scripts']))>0 && trim($this->conf['scripts']) != "scripts/") $this->extManConf['scripts'] = trim($this->conf['scripts']);
		
		if (strlen(trim($this->conf['styles']))>0 && trim($this->conf['styles']) != "styles/") $this->extManConf['styles'] = trim($this->conf['styles']);
	}
	
	/**
	 * Function to get get flexform values
	 *
	 * @return	array contains flex form valuees
	 */
	public function getflexFormValue() {
		$temp = array();
		
		// get flexform values
		$piFlexForm = $this->cObj->data['pi_flexform'];
		$temp['label'] = $this->pi_getFFvalue($piFlexForm, 'cLabel', 'sDEF');
		$temp['lang'] = $this->pi_getFFvalue($piFlexForm, 'cLang', 'sDEF');
		$temp['code'] = $this->pi_getFFvalue($piFlexForm, 'cCode', 'sDEF');
		
		$temp['css'] = array();
		$temp['css']['highlight'] = $this->pi_getFFvalue($piFlexForm, 'cHighlight', 'sOPTIONS');
		$temp['css']['gutter'] = $this->pi_getFFvalue($piFlexForm, 'cGutter', 'sOPTIONS');
		$temp['css']['toolbar'] = $this->pi_getFFvalue($piFlexForm, 'cToolbar', 'sOPTIONS');
		$temp['css']['collapse'] = $this->pi_getFFvalue($piFlexForm, 'cCollapse', 'sOPTIONS');
		
		return $temp;
	}
	
	/**
	 * Function to make preview in BE FCE Element
	 *
	 */
	public function makePreview() {
		$preview = ($this->flexFormValue['label']) ? "[".$this->flexFormValue['lang']."] ".substr($this->flexFormValue['label'],0,110) : "[".$this->flexFormValue['lang']."] ".htmlspecialchars(substr($this->flexFormValue['code'],0,120));
		$bodytext = $this->cObj->data['bodytext'];
		if($bodytext != $preview) {
			#copy the code to bodytext for preview in BE
			$res = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tt_content','uid='.$this->cObj->data['uid'],array('bodytext'=>$preview)); 
		}
	}
	
	/**
	 * Function to solve CSSconfiguration which overwrites TS configuration 
	 *
	 * @return	string  space seperated CSS classes
	 */
	public function getCssConfig() {
		// built brushes string
		$string = '';
		$highlight = array();
		foreach($this->flexFormValue['css'] AS $config => $configValue) { 
			if ($configValue != "" && $configValue != "auto") {
				// highlight range
				if ($config == "highlight") {
					$string .= " boc-highlight[".t3lib_div::expandList($configValue)."]";
				} else {
					if ($configValue) $string .= " boc-".$config;
					else $string .= " boc-no-".$config;
				}
			}
		}
		return $string;
	}
	
	/**
	 * Function to solve brushes 
	 *
	 * @return	string  comma seperated list of the brushes with double quotes
	 */
	public function getBrushes() {
		// built brushes string
		$temp = '';
		if (strlen($this->conf['brushes'])>0) {
			$brushesArray = t3lib_div::trimExplode(",", $this->conf['brushes'], true);
			foreach($brushesArray AS $brush) { 
				$temp .= ',"'.$brush.'"';			
			}
		}
		return $temp;
	}
	
	/**
	 * Function to solve defaults 
	 *
	 * @return	string  comma seperated list list of the brushes with double quotes with no ending ,
	 */
	public function getDefaults() {
		$temp = '';
		if (is_array($this->conf['defaults.'])>0) {
			foreach($this->conf['defaults.'] AS $key => $value) { 
				$temp .= '"'.$key.'": '.$value.',';			 
			} 
			$temp = substr($defaults, 0, -1);
		}		
		return $temp;
	}
	
	/**
	 * Function to solve language strings 
	 *
	 * @return	string  object with language strings
	 */
	public function getLanguageStrings() {
		$temp = '';
		if (is_array($this->conf['config.']['strings.'])>0) {
			$temp .= "\n\t\t\t".'strings: {'."\n";
			foreach($this->conf['config.']['strings.'] AS $key => $value) { 
				$temp .= "\t\t\t\t".trim($key).': "'.trim($value).'",'."\n";
			} 
			$temp .= "\t\t\t".'}'."\n";
		}		
		return $temp;
	}
	
	/**
	 * Function to solve path with FILE: and EXT: 
	 *
	 * @param	string	path to directory
	 * @return	string
	 */
	public function makeAbsolutePath($dir) {
		if (t3lib_div::isFirstPartOfStr($dir, 'EXT:'))	{
			list($extKey,$script)=explode('/',substr($dir,4),2);
			if ($extKey && t3lib_extMgm::isLoaded($extKey))	{
				$extPath=t3lib_extMgm::extPath($extKey);
				return substr($extPath,strlen(PATH_site)).$script;
			}
		} elseif (t3lib_div::isFirstPartOfStr($dir, 'FILE:')) {
				return substr($dir,5);					
		} else {
			return $dir;
		}	
	}
	
	/**
	 * Function to fetch the template file
	 *
	 */
	public function getTemplateFile() {		
		// Get the template
		$templateFile = (strlen(trim($this->conf['templateFile']))>0) ? trim($this->conf['templateFile']) : "EXT:beautyofcode/res/template.html";
		$this->templateHtml = $this->cObj->fileResource($templateFile);
		if (!$this->templateHtml) $this->handleError('Error while fetching the template file: <em>'.$templateFile.'</em>');				
	}
	
	/**
	 * Function to check if static template is included
	 *
	 * @return	string or boolean
	 */
	public function checkForStaticTempl() {
		if (is_array($this->conf)
			&& !isset($this->conf["noConflict"])
			&& !isset($this->conf["addjQuery"])
			&& !isset($this->conf["showLabel"])
			&& !isset($this->conf["theme"])
			&& !isset($this->conf["brushes"])
			&& !isset($this->conf["defaults"])
		) {
			$this->handleError('Please add the static TS to your main template.');
		}
	}
	
	/**
	 * Handles error output for frontend and TYPO3 logging
	 *
	 * @param	string	Message to output
	 * @return	void
	 * @see	t3lib::devLog()
	 * @see	t3lib_div::sysLog()
	 */
	public function handleError($msg) {
		// prepare FE output
		if ($this->error === false) $this->error = array();			
		$this->error[] = $msg . "<br />";
		
		t3lib_div::sysLog($msg, $this->extKey, 3); // error
		// write dev log if enabled
		if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['enable_DLOG']) {
			t3lib_div::devLog($msg, $this->extKey, 3); // fatal error
		}
	}
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/beautyofcode/pi1/class.tx_beautyofcode_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/beautyofcode/pi1/class.tx_beautyofcode_pi1.php']);
}

?>